/*
 * Copyright 2002-2007 Interface21.
 * This code is free to use, modify and extend.
 * Out of courtesy rather than legal obligation,
 * please acknowledge its source if reproducing
 * it!
 */

package com.interface21.spring2.aop;

import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;

import com.interface21.spring2.ioc.Person;

/**
 * Example of @AspectJ style aspect model. 
 * The pointcut expression is included in the annotation.
 * The method itself constitutes the advice.
 * <br>
 * Note that the annotations are in the aspect,
 * <b>not</b> the mainline code, so we don't need
 * to have annotations "crosscutting" our business logic.
 * 
 * @author Rod Johnson
 * @since 2.0
 */
@Aspect
public class AnnotatedBirthdayCardSender {
	
	@After("execution(void com.interface21..Person.birthday()) and this(person)")
	public void onBirthday(Person person) {
		System.out.println("I will send a birthday card to " +
				person.getName() + "; he has just turned " + 
				person.getAge());
	}

}
