/*
 * Copyright 2002-2007 Interface21.
 * This code is free to use, modify and extend.
 * Out of courtesy rather than legal obligation,
 * please acknowledge its source if reproducing
 * it!
 */

package com.interface21.spring2.aop;


import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;

/**
 * Example of @AspectJ style pointcut definition and reuse.
 * Reuse is always best practice--and as pointcuts are 
 * first class citizens in AspectJ/Spring 2.0 AOP, it's
 * good practice to reuse them, avoid duplication and
 * create a language of architectural structure!
 * 
 * The @Pointcut annotation can be attached to a method.
 * Any argument binding will be defined by method parameters.
 * @author Rod Johnson
 * @since 2.0
 */
@Aspect
public class PointcutReuse {
	
	@Pointcut("execution(public !void get*())")
	public void getter() {}
	
	@Pointcut("execution(public int *())")
	public void methodReturningInt() {}
	
	@Pointcut("execution(public void *(..))")
	public void voidMethod() {}
	
	@Pointcut("execution(public * *())")
	public void takesNoArgs() {}
	
	@After("methodReturningInt()")
	public void returnedInt(JoinPoint jp) {
		System.out.println("Method " + jp.getSignature().getName() + 
				" returned int");
	}
	
	@After("getter()")
	public void getterCalled(JoinPoint jp) {
		System.out.println("Method " + jp.getSignature().getName() + 
				" is a getter");
	}
	
	@After("getter() and methodReturningInt()")
	public void getterCalledThatReturnsInt(JoinPoint jp) {
		System.out.println("ANDing of pointcuts: Method " + jp.getSignature().getName() + 
				" is a getter that also returns int");
	}
	
	@After("getter() or voidMethod()")
	public void getterOrVoidMethodCalled(JoinPoint jp) {
		System.out.println("ORing of pointcuts: Method " + jp.getSignature().getName() + 
				" is a getter OR is void");
	}

}
