/*
 * Copyright 2002-2007 Interface21.
 * This code is free to use, modify and extend.
 * Out of courtesy rather than legal obligation,
 * please acknowledge its source if reproducing
 * it!
 */

package com.interface21.spring.aop;

import org.springframework.aop.support.AopUtils;
import org.springframework.test.AbstractDependencyInjectionSpringContextTests;

import com.interface21.spring2.ioc.Person;

/**
 * Integration test superclass using the org.springframework.test package 
 * (included in spring-mock.jar) to dependency inject the
 * test fixture and drive AOP tests.
 * 
 * @author Rod Johnson
 * @since 2.0
 */
public abstract class AbstractAopTest extends AbstractDependencyInjectionSpringContextTests {

	protected Person person;

	/**
	 * Injected by test superclass. This corresponds to a single
	 * bean of this type defined in application context.
	 * The test superclass uses autowire by type by default.
	 * @param person the person to set
	 */
	public void setPerson(Person person) {
		this.person = person;
	}

	public void testIsProxied() {
		assertTrue(AopUtils.isAopProxy(person));
	}

	public void testBirthday() {
		person.birthday();
	}

}