/*
 * Copyright 2002-2007 Interface21.
 * This code is free to use, modify and extend.
 * Out of courtesy rather than legal obligation,
 * please acknowledge its source if reproducing
 * it!
 */

package com.interface21.spring.aop;

import java.lang.reflect.Method;

import junit.framework.TestCase;

import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springframework.aop.MethodBeforeAdvice;
import org.springframework.aop.aspectj.annotation.AspectJProxyFactory;
import org.springframework.aop.framework.ProxyFactory;

import com.interface21.spring2.ioc.Person;

/**
 * @author Rod Johnson
 * @since 2.0
 */
public class ParameterBindingTest extends TestCase {
	
	public void testAspectJParameterBinding() {
		Person tony = new Person();
		tony.setName("Tony");
		tony.setAge(53);
		
		AspectJProxyFactory ajpf = new AspectJProxyFactory(tony);
		ajpf.addAspect(new ParameterCaptureAspect());
		Person proxy = ajpf.getProxy();
		
		proxy.setName("Greg");
	}
	
	public void testSpring1xParameterBinding() {
		Person tony = new Person();
		tony.setName("Tony");
		tony.setAge(53);
		
		ProxyFactory pf = new ProxyFactory(tony);
		pf.addAdvice(new ParameterCaptureInterceptor());
		Person proxy = (Person) pf.getProxy();
		
		proxy.setName("Greg");
	}
	
	@Aspect
	public static class ParameterCaptureAspect {
		
		@Before("execution(* *.*(String, ..)) && args(s)")
		public void logStringArg(String s) {
			System.out.println("String arg was '" + s + "'");
		}
	}
	
	public static class ParameterCaptureInterceptor implements MethodBeforeAdvice {

		public void before(Method method, Object[] args, Object target) throws Throwable {
			if (args.length >= 1 && method.getParameterTypes()[0] == String.class) {
				String s = (String) args[0];
				System.out.println("String arg was '" + s + "'");
			}
		}
	}

}
